@@ -2,7 +2,7 @@
 """
 Control bar for all Iris's controls
 """
-from collections import Iterable
+from collections.abc import Iterable
 
 from PyQt5 import QtCore, QtGui, QtWidgets
 from pywt import Modes
@@ -384,7 +384,7 @@ class PowderDiffractionDatasetControl(QtWidgets.QFrame):
         self.resize(self.minimumSize())
 
     def baseline_parameters(self):
-        """ Returns a dictionary of baseline-computation parameters """
+        """Returns a dictionary of baseline-computation parameters"""
         return {
             "first_stage": self.first_stage_cb.currentText(),
             "wavelet": self.wavelet_cb.currentText(),
